import 'package:dio/dio.dart';

import 'ResultObject.dart';

class CommonApp {
  /**
   * function public
   */
  static Future<ResultObject> loadDataJsonResult(String url,
      {bool methodGet = true,
      Map<String, dynamic> queryParamaters,
      dynamic data}) async {
    try {
      var response = await loadDataJson(url,
          methodGet: methodGet, queryParamater: queryParamaters, data: data);
      return ResultObject.fromJson(response.data);
    } catch (error) {
      print(
          "Error loadDataJsonResult(url: $url, methodGet :$methodGet,queryParamater: $queryParamaters  ) ${_handleError(error)}");
      var result = new ResultObject();
      result.ErrorMessage =
          "Error loadDataJsonResult(url: $url, methodGet :$methodGet,queryParamater: $queryParamaters  ) ${_handleError(error)}";
      result.IsError = true;
      return result;
    }
  }

/**
 * function private
 */
  static Future<Response> loadDataJson(String url,
      {bool methodGet = true,
      Map<String, dynamic> queryParamater,
      Map<String, dynamic> headers,
      dynamic data}) async {
    BaseOptions options =
        BaseOptions(receiveTimeout: 5000, connectTimeout: 5000);
    if (headers != null) options.headers = headers;
    Dio dio = Dio(options);
    Response response;
    try {
      if (methodGet)
        response = await dio.get(url, queryParameters: queryParamater);
      else
        response = await dio.post(url, data: data);
      return response;
    } catch (error) {
      throw {error};
    } finally {
      dio.close();
      response = null;
    }
  }

  static String _handleError(Error error) {
    String errorDescription = "";
    if (error is DioError) {
      DioError dioError = error as DioError;
      switch (dioError.type) {
        case DioErrorType.CANCEL:
          errorDescription = "Request to API server was cancelled";
          break;
        case DioErrorType.CONNECT_TIMEOUT:
          errorDescription = "Connection timeout with API server";
          break;
        case DioErrorType.DEFAULT:
          errorDescription =
              "Connection to API server failed due to internet connection";
          break;
        case DioErrorType.RECEIVE_TIMEOUT:
          errorDescription = "Receive timeout in connection with API server";
          break;
        case DioErrorType.RESPONSE:
          errorDescription =
              "Received invalid status code: ${dioError.response.statusCode}";
          break;
        case DioErrorType.SEND_TIMEOUT:
          errorDescription = "Send timeout in connection with API server";
          break;
      }
    } else {
      errorDescription = "Unexpected error occured";
    }
    return errorDescription;
  }
}

class LoggingInterceptor extends Interceptor {
  int _maxCharactersPerLine = 200;

  @override
  Future onRequest(RequestOptions options) {
    return super.onRequest(options);
  }

  @override
  Future onResponse(Response response) {
    print(
        "onResponse ${response.statusCode} ${response.request.method} ${response.request.path}");
    String responseAsString = response.data.toString();
    if (responseAsString.length > _maxCharactersPerLine) {
      int iterations =
          (responseAsString.length / _maxCharactersPerLine).floor();
      for (int i = 0; i <= iterations; i++) {
        int endingIndex = i * _maxCharactersPerLine + _maxCharactersPerLine;
        if (endingIndex > responseAsString.length) {
          endingIndex = responseAsString.length;
        }
        print(
            responseAsString.substring(i * _maxCharactersPerLine, endingIndex));
      }
    } else {
      print(response.data);
    }
    print("<-- END HTTP");

    return super.onResponse(response);
  }

  @override
  Future onError(DioError err) {
    print("Error: ${err.error} Message: ${err.message}");
    return super.onError(err);
  }
}
