import 'package:flutter/foundation.dart';

import 'Exceptions/NoConnectionException.dart';
import 'IReponsitory.dart';

class HiveRepository<T> implements IRepository<T> {
  final IRepository<T> source;
  final IRepository<T> cache;
  final bool Function() hasConnection;
  HiveRepository({
    @required this.source,
    @required this.cache,
    @required this.hasConnection,
  });

  @override
  Future<T> get(dynamic id) async {
    final cacheData = await this.cache.get(id);
    if (cacheData != null) return cacheData;

    if (!this.hasConnection()) throw NoConnectionException();

    final remoteData = await this.source.get(id);
    this.cache.add(remoteData);

    return remoteData;
  }

  @override
  Future<void> add(T object) async {
    if (!this.hasConnection()) throw NoConnectionException();

    await this.source.add(object);
    await this.cache.add(object);
  }

  @override
  Future<void> addAll(Iterable<T> object) async {
    if (!this.hasConnection()) throw NoConnectionException();
    await this.source.addAll(object);
    await this.cache.addAll(object);
  }

  @override
  Future<void> delete(dynamic id) async {
    if (!this.hasConnection()) throw NoConnectionException();
    var item = await get(id);
    if (item != null) {
      await this.source.delete(id);
      await this.cache.delete(id);
    }
  }

  @override
  Future<void> update(key, T object) async {
    if (!this.hasConnection()) throw NoConnectionException();
    var item = await get(key);
    if (item == null) {
      await this.source.update(key, object);
      await this.cache.update(key, object);
    }
  }
}
